#ifndef __MY_TYPES__
#define __MY_TYPES__

/*
64 bit machine?
*/
#if defined(__LP64__) || defined(_LP64) || defined(__64BIT__) || defined(__powerpc64__) \
 || defined(_WIN64) || defined(_M_IA64) || (defined(_MIPS_SZLONG) && _MIPS_SZLONG == 64)
#    define ARC_64BIT
#endif
/*
int types
*/
#ifdef _MSC_VER
     typedef __int8 int8_t;
     typedef unsigned __int8 uint8_t;
     typedef __int16 int16_t;
     typedef unsigned __int16 uint16_t;
     typedef __int32 int32_t;
     typedef unsigned __int32 uint32_t;
     typedef __int64 int64_t;
     typedef unsigned __int64 uint64_t;
#else
#    include <inttypes.h>
#endif

typedef int8_t BMP8;
typedef uint8_t UBMP8;
typedef int16_t BMP16;
typedef uint16_t UBMP16;
typedef int32_t BMP32;
typedef uint32_t UBMP32;
typedef int64_t BMP64;
typedef uint64_t UBMP64;

/*
Os stuff
*/
#ifdef _MSC_VER
#    include <windows.h>
#    include <io.h>
#    include <conio.h>
#    undef CDECL
#    define CDECL __cdecl
#    define UINT64(x) (x##ui64)
#    define FMT64     "%016I64x"
#    define FORCEINLINE __forceinline
#else
#    include <unistd.h>
#    include <dlfcn.h>
#    define CDECL
#    ifdef ARC_64BIT
#        define UINT64
#    else
#        define UINT64(x) (x##ULL)
#    endif
#    define FMT64     "%016llx"
#    define FORCEINLINE __inline
#endif
/*
Define PARALLEL to enable multi threading
*/
#define PARALLEL
/* From http://dshawul.googlepages.com/home:
d) If you want to compile the egbbdlls for Linux, make sure you comment
#define USESPINLOCK in my_types.h (line 66). Spinlock implementation is broken which
leads to segmentation fault.
*/
//#define USESPINLOCK

/*smp optional code*/
#ifdef PARALLEL
#	 define SMP_CODE(x) x
#else
#    define SMP_CODE(x)
#endif

/*modify these to enable smp debugging codes.*/
#define SMP_DEBUG(x)
#define SMP_DEBUG1(x)


#ifdef PARALLEL
#    define VOLATILE volatile
/*threads*/
#    ifdef _MSC_VER
#        include <process.h>
#        define pthread_t HANDLE
#        define t_create(f,p,t) t = (pthread_t) _beginthread(f,0,(void*)p)
#        define t_sleep(x)    Sleep(x)
#    else
#        include <pthread.h>
#        define t_create(f,p,t) pthread_create(&t,0,(void*(*)(void*))&f,(void*)p)
#        define t_sleep(x)    usleep((x) * 1000)
#    endif
/*locks*/
#    ifdef _MSC_VER
#        ifdef USESPINLOCK
#             define LOCK VOLATILE int
#             define l_create(x)   ((x) = 0)
#             define l_lock(x)     while(InterlockedExchange((LPLONG)&(x),1) != 0) {while((x) != 0);}
#             define l_unlock(x)   ((x) = 0)
#        else
#             define LOCK CRITICAL_SECTION
#             define l_create(x)   InitializeCriticalSection(&x)
#             define l_lock(x)     EnterCriticalSection(&x)
#             define l_unlock(x)   LeaveCriticalSection(&x)
#        endif   
#    else
#        ifdef USESPINLOCK
#             define LOCK VOLATILE int
#             define l_create(x)   ((x) = 0)
#			  define l_lock(x) {				\
	int dummy;									\
	asm __volatile__ (							\
		"1:          movl    $1, %0" "\n\t"		\
		"            xchgl   (%1), %0" "\n\t"	\
		"            testl   %0, %0" "\n\t"		\
		"            jz      3f" "\n\t"			\
		"2:          pause" "\n\t"				\
		"            movl    (%1), %0" "\n\t"	\
		"            testl   %0, %0" "\n\t"		\
		"            jnz     2b" "\n\t"			\
		"            jmp     1b" "\n\t"			\
		"3:" "\n\t"								\
		:"=&q" (dummy)							\
		:"q" (x)								\
		:"cc", "memory");						\
}
#             define l_unlock(x)   ((x) = 0)
#        else
#			  define LOCK pthread_mutex_t
#			  define l_create(x)   pthread_mutex_init(&(x),0)
#			  define l_lock(x)     pthread_mutex_lock(&(x))
#			  define l_unlock(x)   pthread_mutex_unlock(&(x))
#        endif
#    endif
#else
#    define VOLATILE
#    define l_lock(x)
#    define l_unlock(x)
#endif
/*
end
*/
#endif
